@@ -18,88 +18,64 @@ import pytz
 import pandas as pd
 import logbook
 
-from intuition.data.utils import Exchanges
-from intuition.modules.sources.loader import LiveBenchmark
-from intuition.core.analyzes import Analyze
-
 from zipline.finance.trading import TradingEnvironment
 from zipline.utils.factory import create_simulation_parameters
 
-import intuition.modules.library as library
+import intuition.constants as constants
+from intuition.data.utils import Exchanges
+from intuition.data.loader import LiveBenchmark
+from intuition.core.analyzes import Analyze
+import intuition.utils.utils as utils
 
 
-BASE_CONFIG = {'algorithm': {}, 'manager': {}}
-DEFAULT_PORTFOLIO_NAME = 'ChuckNorris'
 log = logbook.Logger('intuition.core.engine')
 
 
+def _intuition_module(location):
+    ''' Build the module path and import it '''
+    path = location.split('.')
+    obj = path.pop(-1)
+    return utils.dynamic_import('.'.join(path), obj)
+
+
+#NOTE Is there still a point to use here a constructor object instead of a
+#     simple function ?
 class TradingEngine(object):
     ''' Factory class wrapping zipline Backtester, returns the requested algo
     ready for use '''
 
-    def __new__(self, algo, manager=None, source=None,
-                strategy_configuration=BASE_CONFIG):
-        '''
-        Reads the user configuration and returns
-        '''
-        library.check_availability(algo, manager, source)
+    def __new__(self, identity, modules,
+                strategy_conf=constants.DEFAULT_CONFIG):
 
-        #NOTE Other params: annualizer (default is cool), capital_base,
-        #     data_frequency, sim_params (both are set in run function)
-        trading_algorithm = library.algorithms[algo](
-            properties=strategy_configuration['algorithm'])
+        algo_obj = _intuition_module(modules['algorithm'])
+        algo_obj.identity = identity
+        trading_algo = algo_obj(properties=strategy_conf['algorithm'])
 
-        portfolio_name = strategy_configuration['manager'].get(
-            'name', DEFAULT_PORTFOLIO_NAME)
-        trading_algorithm.set_logger(logbook.Logger('algo.' + portfolio_name))
+        trading_algo.set_logger(logbook.Logger('algo.' + identity))
 
-        if source:
-            trading_algorithm.set_data_generator(library.data_sources[source])
+        if modules['data']:
+            trading_algo.set_data_generator(_intuition_module(modules['data']))
 
-        # Use of a portfolio manager
-        if manager:
-            log.info('initializing Manager')
+        # Use a portfolio manager
+        if modules['manager']:
+            log.info('initializing manager {}'.format(modules['manager']))
             # Linking to the algorithm the configured portfolio manager
-            trading_algorithm.manager = library.portfolio_managers[manager](
-                strategy_configuration['manager'])
-
-            # If requested and possible, loads the named portfolio to start
-            # trading with it
-            #FIXME Works, but every new event resets the portfolio
-
-            if strategy_configuration['manager'].get('load_backup', False) \
-                    and portfolio_name:
-
-                log.info('Re-loading last {} portfolio from database'
-                         .format(portfolio_name))
-                # Retrieving a zipline portfolio object. str() is needed as the
-                # parameter is of type unicode
-                backup_portfolio = trading_algorithm.manager.load_portfolio(
-                    str(portfolio_name))
-
-                if backup_portfolio is None:
-                    log.warning('! Unable to set {} portfolio: not found'
-                                .format(portfolio_name))
-                else:
-                    trading_algorithm.set_portfolio(backup_portfolio)
-                    log.info('Portfolio setup successful')
+            trading_algo.manager = _intuition_module(modules['manager'])(
+                strategy_conf['manager'])
         else:
-            trading_algorithm.manager = None
+            trading_algo.manager = None
             log.info('no portfolio manager used')
 
-        return trading_algorithm
+        return trading_algo
 
 
-#NOTE engine.feed_data(universe, start, end, freq) ? using set_source()
 class Simulation(object):
     ''' Take a trading strategy and evalute its results '''
 
     def __init__(self, configuration):
-        #self.server        = ZMQ_Dealer(id=self.__class__.__name__)
         self.configuration = configuration
         self.context = None
 
-    #TODO For both, timezone configuration
     def configure(self):
         '''
         Prepare dates, data, trading environment for simulation
@@ -115,17 +91,13 @@ class Simulation(object):
         self.context = self._configure_context(self.configuration['exchange'])
 
     def set_benchmark_loader(self, load_function):
+        ''' Define a custom benchmark loader for zipline to use '''
         self.load_market_data = load_function
 
-    #TODO Use of futur localisation database criteria
     def _configure_context(self, exchange=''):
         '''
         Setup from exchange traded on benchmarks used, location
         and method to load data market while simulating
-        _______________________________________________
-        Parameters
-            exchange: str
-                Trading exchange market
         '''
         # Environment configuration
         if exchange in Exchanges:
@@ -139,10 +111,11 @@ class Simulation(object):
 
         return trading_context
 
-    def run(self, data, strategy):
-        engine = TradingEngine(self.configuration['modules']['algorithm'],
-                               self.configuration['modules']['manager'],
-                               self.configuration['modules']['data'],
+    def run(self, identity, data, strategy):
+        ''' Wrapper of zipline run() method. Use the configuration set so far
+        to build up the trading environment and launch the system '''
+        engine = TradingEngine(identity,
+                               self.configuration['modules'],
                                strategy)
 
         #NOTE This method does not change anything
@@ -160,10 +133,9 @@ class Simulation(object):
                 start=self.configuration['index'][0],
                 end=self.configuration['index'][-1])
 
-            daily_stats = engine.go(data, sim_params=sim_params)
+            daily_stats = engine.trade(data, sim_params=sim_params)
 
         return Analyze(
             results=daily_stats,
             metrics=engine.risk_report,
-            datafeed=None,
             configuration=self.configuration)
